class Admin::TagsController < OrbitAdminController
  before_filter :setup_vars

  def index
    @tag = Tag.new
    @tags = @is_module_tag ? @module_app.tags.where(:is_default=>true) : Tag.where(:is_default=>true)
    @default_tags = Tag.where(:is_default=>true).not_in(:id => @module_app.tag_ids)
  end

  def create
      @tag = Tag.new(tag_params.merge(is_default: !@is_module_tag))
      if @tag.save
        @module_app.tags << @tag  if @is_module_tag
      end
      redirect_to request.referer, :status => 303
  end

  def create_in_form
    @tag = Tag.new(tag_params.merge(is_default: !@is_module_tag))
    @tag.save
    @module_app.tags << @tag
    render :json => {:id=>@tag.id.to_s, :name=>@tag.name}.to_json ,:layout => false
  end

  def update
    @tag = Tag.find(params[:id])
    if @tag.update_attributes(tag_params)
      redirect_to request.referer, :status => 303
    else
      flash.now[:error] = t('update.error.tag')
      render :action => "edit"
    end
  end

  def delete_tags
    tags = Tag.find(params[:ids].split(',')) rescue nil
    if tags
      tags.each(&:destroy)
    end
    redirect_to request.referer, :status => 303
  end

  def add_from_default
    tags = Tag.find(params[:ids].split(',')) rescue nil
    if tags
      @module_app.tags << tags
    end
    redirect_to request.referer, :status => 303
  end

  def add_to_default
    tags = Tag.find(params[:ids].split(',')) rescue nil
    if tags
      tags.each do |tag|
        tag.update_attribute(:is_default, true)
      end
    end
    redirect_to request.referer, :status => 303
  end

  def remove_default
    tags = Tag.find(params[:ids].split(',')) rescue nil
    if tags
      tags.each do |tag|
        tag.update_attribute(:is_default, false)
      end
    end
    redirect_to request.referer, :status => 303
  end

  def merge
    tags = Tag.find(params[:ids])
    new_tag = Tag.new tag_params

    mods = []

    tags.each do |tag|
      new_tag.is_default = true if tag.is_default

      tag.taggings.each do |tagging|
        taggable = tagging.taggable
        taggable.tags = taggable.tags - [tag] + [new_tag]
        taggable.save
      end

      unless tag.module_app.nil? 
        tag.module_app.each do |mod|
          mods << mod
          mod.tags = mod.tags - [tag] + [new_tag]
          mod.save
        end  
      end
      
      tag.destroy
    end

    new_tag.is_default = true if mods.flatten.uniq.size > 1

    new_tag.save

    redirect_to request.referer, :status => 303
  end

  def tag_params
    params.require(:tag).permit! unless params[:tag].blank?
  end

  protected
  
  def setup_vars
    @is_module_tag = !params[:module_app_id].blank?
    @module_apps = @is_module_tag ? [ModuleApp.find(params[:module_app_id])] : ModuleApp.where(taggable: true)
    @module_app = @is_module_tag ? ModuleApp.find(params[:module_app_id]) : ModuleApp.find_by(:key=>'tag')
  end

end