module OrbitTag
  module Taggable

    def self.included(base)
      base.has_many :taggings, as: :taggable, autosave: true, dependent: :destroy
      base.accepts_nested_attributes_for :taggings, allow_destroy: true
      base.extend(ClassMethods)
    end

    module ClassMethods
      def with_tags(tags=[])
        tags = [tags].flatten.uniq
        if tags.blank?
          self.all
        else
          taggings = Tagging.where(:tag_id.in=>tags).map{|item| item.taggable_id}
          self.where(:id.in=>taggings)
        end
      end

      def filter_by_tags(tags=[])
        tags = OrbitHelper.page_tags if tags.blank?
        tags = [tags].flatten.uniq
        if tags.blank? or (tags.include?("all") rescue false)
          self.all
        else
          taggings = Tagging.where(:tag_id.in=>tags).map{|item| item.taggable_id}
          self.where(:id.in=>taggings)
        end
      end

    end

    def tags
      self.taggings.blank? ? [] : self.taggings.map{|t| t.tag}.compact
    end

    def tags=(tags)
      tags = [tags].flatten.uniq
      Tagging.where(:taggable=>self.id).destroy_all
      tags.each do |tag|
        self.taggings.build(tag: tag)
      end
    end

    def tag_ids
      self.taggings.blank? ? [] : self.taggings.map{|t| t.tag.id}.compact
    end

  end
end